#########################################################################################
#                                Lamb's WTs with ERA-Interim                            #
#########################################################################################
# run it from the bash with:
#
# Rscript WT_v5.R 1984
#
# being 1984 the year you want to classify the WTs;
# in this way you can run up to 8 jobs at the same time, each one producing its output files!
# You can also run it for a sequence of years with the syntax:
# 
# Rscript WT_v5.R 1980 2014
#
# and it will compute each year from 1980 to 2014, each one after finishing the previous one.
#
# If you want to run many years in parallel with just only 1 command, run from the bash:
#
# for y in {1980..2014}; do Rscript WT_v5.R &; done
#
# but it'd need 24 processors! In practice, it is possible to run only 4-8 years at time.
#

library(s2dverification)  # for the function Load()
# library(ff)
source('/home/Earth/ncortesi/scripts/Rfunctions.R') # for the calendar functions
#Load.path <- '/home/Earth/ncortesi/Downloads/scripts/IC3.conf'      # Load() file path

# Available reanalysis:
ERAint <- list(path = '/esnas/recon/ecmwf/erainterim/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc')
JRA55 <- list(path = '/esnas/recon/jma/jra-55/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc')

workdir="/scratch/Earth/ncortesi/RESILIENCE/WT"    # working dir where to put the output maps and files
#subdatadir="/scratch/Earth/ncortesi/RESILIENCE/WT_subdata"    # dir where to put the intermediate daily psl data (1 file for each subarray)

mslp.rean=ERAint #'ERAintDailyHighRes'  #'ERAintDailyLowRes'  # choose one of the daily reanalysis above for loading MSLP data
rean.name <- 'erai'

year.start=1981     # starting year of the MSLP daily data (from the 1st of january)
year.end=2016       # ending year of the MSLP daily data (up to the 31 of December)

res=4.9 #5.25       # spacing between the Lamb grid points in the meridional direction (in the zonal direction, it is exactly the double of this value)
                    # it should be put equal to the multiple of 'psl.res' closer to 5 degrees, i.e: res = psl.res * round((5/psl.res))

low.res.size=7      # odd numb. of MSLP nearby grid points to compute the psl at each of the 16 points (both for lon and lat: the actual n.of points used is its square)

#partial.end=FALSE    # put TRUE if the last year ('year.end') has not all the yearly data but stop before December the 31th; in this case, must also specify the variable below
#n.days.last=334     # number of days available in the last year (used only if partial.end=TRUE) I.e: data for 2015 doesn't have December, so it has 365-31=334 days

merge=FALSE         # put TRUE if you want to concatenate all the WT classification for different years and same grid point at the end of the analysis, FALSE otherwise

##########################################################################################

args <- commandArgs(TRUE)

if(length(args) == 1) year.start <- year.end <- as.integer(args[1])
if(length(args) == 2) {year.start <- as.integer(args[1]); year.end <- as.integer(args[2])}

n.years <- year.end - year.start + 1

WTs.type<-c("NE","E","SE","S","SW","W","NW","N","C","C.NE","C.E","C.SE","C.S","C.SW","C.W","C.NW","C.N","A","A.NE","A.E","A.SE","A.S","A.SW","A.W","A.NW","A.N")
radius <- (low.res.size-1)/2  # number of grid points to use as search radius for averaging the psl at each Lamb grid point

# Load 1 day of MSLP data from the reanalysis chosen, just to detect the number of latitude and longitude points:
#MSLP <- Load('psl', NULL, mslp.rean, paste0(year.start,'0101'), storefreq = 'daily', leadtimemax = 1, output = 'lonlat', configfile = Load.path)
MSLP <- Load(var = 'psl', exp = NULL, obs = list(mslp.rean), paste0(year.start,'0101'), storefreq = 'daily', leadtimemax = 1, output = 'lonlat', nprocs=1)

lat <- round(MSLP$lat,3)
lon <- round(MSLP$lon,3)

n.lat <- length(lat)  # number of latitude values
n.lon <- length(lon)  # number of longitude values

lon.pos <- ifelse(min(MSLP$lon>=0), TRUE, FALSE)  # set lon.pos to TRUE if the longitude values of the reanalysis grid have no negative values, FALSE if not.

# for each grid point of the reanalysis, compute the Lamb WT classification with Lamb grid centered on that point.
# we exclude only central points > +77 degrees lat and < -77 deg. because the Lamb grid needs 12.5 deg. north/south of the central point.
n.lat.unused.poles <- 17 #20 # number of unused latitude values near each of the two poles (must exclude last 10 degrees N/S, so it depends on the spatial res. of the reanalysis)

lat.used <- MSLP$lat[-c(1:n.lat.unused.poles,(n.lat-n.lat.unused.poles):n.lat)] # latitude values used as central points 
psl.res <- diff(lat.used)[1]  # psl grid resolution
lat.used <- round(lat.used,3) # round psl values to the third decimal to save them in a file with a short file name (precision: ~100 m)
n.lat.used <- length(lat.used)
    
lon.used <- round(MSLP$lon,3) # longitude values used as central points rounded to the third decimal (precision: ~100 m)
n.lon.used <- length(lon.used)

n.grid.points <- length(lat.used)*length(lon.used)

# save metadata on the WT classification used to retrieve in subsequent analysis (do not save them if you are only adding a new year to an existing classification!!!):
save.image(file=paste0(workdir,"/txt/",rean.name,"/metadata.RData"))

# load all MSLP data one year at time:<<
for(y in year.start:year.end){
       print(paste0("Year: ",y))

       #for(month in 1:12){
       #   print(paste0("Month: ",month))

       #   month_2_digits <- ifelse(month<10, paste0("0",month), month)
       n.days <- n.days.in.a.year(y) #n.days.in.a.month(month,y)

       # load data of year y:
       MSLP.year <- Load(var = 'psl', exp = NULL, obs = list(mslp.rean), paste0(y,'01','01'), storefreq = 'daily', leadtimemax = n.days, output = 'lonlat', nprocs=1)
       gc()

       # loop over the lat/lon of the central point of the Lamb classification:
       for(latc in lat.used){
           #latc <- lat.used[1]; lonc <- lon.used[1];  # for the debug

           ## first, it computes the latitude coefficients:
           SF1 <- 1/cos(latc*pi/180) 
           ZS1 <- 1/(2*cos(latc*pi/180)^2)
           ZW1 <- sin(latc*pi/180)/sin((latc-res)*pi/180)
           ZW2 <- sin(latc*pi/180)/sin((latc+res)*pi/180)
           
           for(lonc in lon.used){
               pos.latc <- which(lat.used==latc)
               pos.lonc <- which(lon.used==lonc)

               n.point <- (pos.latc-1)*n.lon.used + pos.lonc # number of points already computed
               cat(paste0("Grid point saved: ", n.point ,"/", n.grid.points, " "), "\r")
               
               # assign the lat and lon values of the 16 grid points of the Lamb grid centered over the central point:
               lat <- lon <- c()  # lat and lon of the 16 points of Lamb's grid
               lat[1] <- lat[2] <- latc + 2*res
               lat[3] <- lat[4] <- lat[5] <- lat[6] <- latc + res
               lat[7] <- lat[8] <- lat[9] <- lat[10] <- latc
               lat[11] <- lat[12] <- lat[13] <- lat[14] <- latc - res
               lat[15] <- lat[16] <- latc - 2*res

               lon[1] <- lon[4] <- lon[8] <- lon[12] <- lon[15] <- lonc - res
               lon[3] <- lon[7] <- lon[11] <- lonc - 3*res
               lon[2] <- lon[5] <- lon[9] <- lon[13] <- lon[16] <- lonc + res
               lon[6] <- lon[10] <- lon[14] <- lonc + 3*res

               if(lat[1] > 90) stop("*** Lamb grid is employing points beyond 90 degrees ***")
               
                # longitude correction for reanalysis with positive-only longitudes (to stay always positive):
               if(lon.pos){
                   if(lonc - res < 0){
                       lon[1] <- lon[4] <- lon[8] <- lon[12] <- lon[15] <- lonc - res + 360
                       lon[3] <- lon[7] <- lon[11] <- lonc - 3*res + 360
                   }
                   if(lonc - 3*res < 0 && lon - res >= 0){
                       lon[3] <- lon[7] <- lon[11] <- lonc - 3*res + 360
                   }
                   if(lonc + res >= 360){
                       lon[2] <- lon[5] <- lon[9] <- lon[13] <- lon[16] <- lonc + res - 360
                       lon[6] <- lon[10] <- lon[14] <- lonc + 3*res - 360
                   }
                   if(lonc + 3*res >= 360 && lonc + res < 360){
                       lon[6] <- lon[10] <- lon[14] <- lonc + 3*res - 360
                   }
               }

              # longitude correction for reanalysis with [-180, +180] longitude (to stay always in the same interval):
              # MUST STILL BE UPDATED FOR CASE lonc + 15 >= 360 !!!
               if(!lon.pos){
                   if(lonc - res < -180){
                       lon[1] <- lon[4] <- lon[8] <- lon[12] <- lon[15] <- lonc - res + 360
                       lon[3] <- lon[7] <- lon[11] <- lonc - 3*res + 360
                   }
                   if(lonc + res >= 180){
                       lon[2] <- lon[5] <- lon[9] <- lon[13] <- lon[16] <- lonc + res - 360
                       lon[6] <- lon[10] <- lon[14] <- lonc + 3*res - 360
                   }
               }                
               
               # lat and lon position of the 16 Lamb grid points inside the MSLP reanalysis:
               pos.lat <- pos.lon <- c()
               for(p in 1:16){
                   pos <- nearest(lat[p], lon[p], MSLP$lat, MSLP$lon)
                   pos.lat[p] <- pos[1]
                   pos.lon[p] <- pos[2]
               }

               #WTs <- data.frame()
               
               # mean psl at each of the 16 grid points:
               psl <- array(NA,c(n.days,16)) # for each day of the year, MSLP of the 16 points of Lamb's grid
                                        #for(p in 1:16) psl[,p] <- MSLP.year$obs[1,1,1,,pos.lat[p],pos.lon[p]]
               
               for(p in 1:16) {     
                   pos.lon.low.res <- (pos.lon[p]-radius):(pos.lon[p]+radius)
                   
                   if(head(pos.lon.low.res,1) <= 0){
                       ss <- which(pos.lon.low.res <= 0)
                       pos.lon.low.res <- c(pos.lon.low.res[ss] + n.lon.used, pos.lon.low.res[-ss])
                   }
                   if(tail(pos.lon.low.res,1) > n.lon.used){
                       ss <- which(pos.lon.low.res > n.lon.used)
                       pos.lon.low.res <- c(pos.lon.low.res[-ss], pos.lon.low.res[ss]-n.lon.used)
                   }
                   
                   pos.lat.low.res <- (pos.lat[p]-radius):(pos.lat[p]+radius)  # it never has to be corrected because we excluded the poles

                   MSLP.low.res <- MSLP.year$obs[1,1,1,,pos.lat.low.res,pos.lon.low.res]
                   psl[,p] <- apply(MSLP.low.res, 1, mean)
               }

               # compute geostrophical indexes (don't round them to the 3rd decimal number as done by Trigo and Martin-Vide, not to introduce an error that change 3-4% of WTs!!!):
               SF<-WF<-F<-D<-D2<-c()

               WF <- 0.5*(psl[,12]+psl[,13]) - 0.5*(psl[,4]+psl[,5])
               SF <- SF1*(0.25 * (psl[,5]+2*psl[,9]+psl[,13]) - 0.25 * (psl[,4]+2*psl[,8]+psl[,12]))
               F <- (SF*SF + WF*WF)^0.5

               if(latc < 0){
                   WF <- -WF  # because in the southern hemisphere, cyclones and anticyclones spins in the opposite sense!
                   SF <- -SF
               }

               # D e' la direzione del flusso; dato che l'angolo risultato di atan e' espresso in radianti, bisogna convertirlo in gradi moltiplicandolo per (180/pi)
               # D e' l'angolo tra la direzione del vento e la sua proiezione in direzione nord-sud (SF)
               D <- (180/pi) * atan(WF/SF)

               # trasforma l'angolo D iniziale in un angolo D2 sempre positivo che si calcola a partire del Nord andando in senso orario:
               Q1 <- which(WF>0 & SF>0)
               Q2 <- which(WF>0 & SF<=0)
               Q3 <- which(WF<0 & SF<=0)   # calcola quali angoli D appartengono al quadrante 1 (Q1), quali al quadrante 2, quali al 3 e quali al 4.
               Q4 <- which(WF<0 & SF>0)    # il primo quadrante e' in alto a destra, il secondo in basso a destra, il terzo in basso a sinistra, il quarto in alto a sinistra.

               # the wind direction is usually different from the angle D where the wind is blowing to, so a correction has to be made:
               D2 <- D
               D2[Q1] <- D[Q1] + 180  
               D2[Q2] <- D[Q2] + 360
               D2[Q4] <- D[Q4] + 180
               
               # individua i WTs direzionali:
               NE <- which(D2>=22.5 & D2<67.5)
               E <- which(D2>=67.5 & D2<112.5)
               SE <- which(D2>=112.5 & D2<157.5)
               S <- which(D2>=157.5 & D2<202.5)
               SW <- which(D2>=202.5 & D2<247.5)
               W <- which(D2>=247.5 & D2<292.5)
               NW <- which(D2>=292.5 & D2<337.5)
               N <- which(D2>=337.5 | D2<22.5)

               # aggiunge una colonna a destra con il WT direzionale associato all'angolo:
               D2.dir<-rep(NA,length(D2))
               D2.dir[NE]<-"NE";D2.dir[E]<-"E";D2.dir[SE]<-"SE";D2.dir[S]<-"S";D2.dir[SW]<-"SW";D2.dir[W]<-"W";D2.dir[NW]<-"NW";D2.dir[N]<-"N"

               # calcolo componenti vorticita':
               ZS<-ZW<-Z<-c()
               
               ZS <- ZS1*(0.25 * (psl[,6]+2*psl[,10]+psl[,14]) - 0.25 * (psl[,5]+2*psl[,9]+psl[,13]) - 0.25 * (psl[,4]+2*psl[,8]+psl[,12]) + 0.25 * (psl[,3]+2*psl[,7]+psl[,11]))
               ZW <- ZW1 * (0.5*(psl[,15]+psl[,16]) - 0.5*(psl[,8]+psl[,9])) - ZW2 * (0.5*(psl[,8]+psl[,9]) - 0.5*(psl[,1]+psl[,2]))
               Z <- ZS + ZW

               # Pure and hybrid WTs
               cyc.pure <- which(Z>2*F)
               anticyc.pure <- which(Z<(-2*F))
               hybrid.cyc <- which(Z>0 & abs(Z)<2*F & abs(Z)>F)
               hybrid.anticyc <- which(Z<0 & abs(Z)<2*F & abs(Z)>F)
               indeter <- which(F<6 & abs(Z)<6) # tipo di tempo indeterminato (U); the choice of 6 depend on grid size and should be changed if grid res.is higher!

               # create a data frame with all the info about the WT classification:    

               #years.period <- months.period <- days.period <- c()
               #for(y in year.start:year.end) years.period <- c(years.period, rep(y, n.days.in.a.year(y)))
               #for(y in year.start:year.end) months.period <- c(months.period, seq.months.in.a.year(y))
               #for(y in year.start:year.end) days.period <- c(days.period, seq.days.in.a.year(y))
               years.period <- rep(y, n.days)
               months.period <- seq.months.in.a.year(y)[1:n.days]
               days.period <- seq.days.in.a.year(y)[1:n.days]
               WT<-data.frame(Year=years.period, Month=months.period, Day=days.period, Direction=round(D2,3), WT.dir=D2.dir, stringsAsFactors=FALSE)

               WT$Year<-as.numeric(WT$Year);WT$Month<-as.numeric(WT$Month);WT$Day<-as.numeric(WT$Day);WT$Direction<-as.numeric(WT$Direction);WT$WT.dir<-as.character(WT$WT.dir)

               # colonna che identifica quali giorni sono di tipo ibrido e quali di tipo direzionale:
               WT$Hyb <- rep("Directional",length(D2))
               WT$Hyb[cyc.pure] <- "Pure_C"
               WT$Hyb[anticyc.pure] <- "Pure_A"
               WT$Hyb[hybrid.cyc] <- "Hybrid_C"
               WT$Hyb[hybrid.anticyc] <- "Hybrid_A"
               WT$Hyb[which(is.na(D2))] <- NA   # remove the writing "directional" from days with no data
               
               # colonna con la lista dei 26 WTs officiali:
               WT$WT <- WT$D2.dir
               WT$WT[cyc.pure] <- "C"
               WT$WT[anticyc.pure] <- "A"
               WT$WT[hybrid.cyc] <- paste("C.",WT$WT.dir[hybrid.cyc],sep="")
               WT$WT[hybrid.anticyc] <- paste("A.", WT$WT.dir[hybrid.anticyc],sep="")
               #WT$WT[indeter] <- "U"   # comment this line not to include the Unclassified U WT

               WT$WT.num <- match(WT$WT,WTs.type)# aggiunge una colonna con la conversione dei 26 tipi di tempo a numeri da 1 a 26

               # colonna con la lista dei 10 WTs ottenuti unendo a C e ad A gli otto WT direzionali puri;
               # i WTs ibridi si aggiungono ognuno al suo tipo direzionale corrispondente (es: C.SW si aggiunge a SW, ecc.)
               WT$WT10 <- WT$WT.dir
               WT$WT10[cyc.pure] <- "C"
               WT$WT10[anticyc.pure] <- "A"
               WT$WT10.num <- match(WT$WT10,WTs.type) # conversione dei 10 tipi di tempo a numeri da 1 a 9 + il tipo A che si converte nel numero 18

               # add the other geostrophical indexes:
               WT$SF=round(SF,5)
               WT$WF=round(WF,5)
               WT$F=round(F,5)
               WT$ZS=round(ZS,5)
               WT$ZW=round(ZW,5)
               WT$Z=round(Z,5)

               #WTs<-rbind(WTs,WT)               
               #assign(paste0("WT","_year_",y,"_lat_",latc,"_lon_",lonc), WT)

               # when saving as .txt instead as .RData, total computational time decreases from 30h to 12h !!!
               if(dir.exists(paths=paste0(workdir,"/txt/",rean.name,"/",y))) {
                  write.table(WT,paste0(workdir,"/txt/",rean.name,"/",y,"/WT_year_",y,"_lat_",latc,"_lon_",lonc,".txt"))
               } else {
                  dir.create(paste0(workdir,"/txt/",rean.name,"/",y))  # create the dir with the year data if it doesn't exist
               }
                  
               rm(WT,SF,WF,F,D,ZS,ZW,W)
                                   
               gc()            
               
           } # close for on lonc
       } # close for on latc
       #cat("\n")

       rm(MSLP.year)

       # save all classification of a single year and month in a single .RData file:    
       #output <- paste0(workdir,"/WTs_",rean.name,"_year_",y,".RData")
       #save.image(file=output)
       
       gc()
       
    #}  # close for on month
    #gc()
    
} # close for on y 


postproc <- FALSE
if(postproc == TRUE){

# merge all txt data of the same lat and lon in 1 file to have all the years together and delete the yearly files:
# beware that if you do it in an interactive session, time available (12h) is usually not enough to finish this operation!
# this txt files are convenient if you want to give them to other researcher who study a specific area

if(merge){
    if(!dir.exists(file.path(workdir,paste0("txt/ERAint/all_years")))) dir.create(file.path(workdir,paste0("txt/ERAint/all_years")))

    #lat.used <- lat.used[205:221]
    
    for(latc in lat.used){
        #latc <- lat.used[1]; lonc <- lon.used[1];  # for the debug

        for(lonc in lon.used){
            pos.latc <- which(lat.used==latc)
            pos.lonc <- which(lon.used==lonc)
            
            #cat(paste0("Merging classification at grid point: ",(pos.latc-1)*n.lon.used + pos.lonc ,"/", n.grid.points, " "), "\r")
            print(paste0("lat: ",latc," lon: ", lonc))

            for(y in year.start:year.end){
                #print(paste0("Year: ",y))
                
                WTs <- read.table(file=paste0(workdir,"/txt/",rean.name,"/",y,"/WT_year_",y,"_lat_",latc,"_lon_",lonc,".txt"),stringsAsFactors=FALSE, header=TRUE) # Load WTs
                if(y == year.start){
                    WTs_full_period <- WTs
                } else {
                    WTs_full_period <- rbind(WTs_full_period, WTs)
                }

            }

            
            write.table(WTs_full_period, file=paste0(workdir,"/txt/",rean.name,"/all_years/WTs_",year.start,"-",year.end,"_lat_",latc,"_lon_",lonc,".txt"),row.names=FALSE, col.names=TRUE, quote=FALSE, sep=" ")
            
            rm(WTs_full_period)
            
        }
    }

  # delete the old yearly files in the <year>_txt directories and the directories:
  for(y in year.start:year.end){
      system(paste0("rm -fr ",workdir,"/",y,"_txt/"))    
  }
    
} # close if on merge





    
# merge all the txt files together in a single array of format [years, days, lat, lon]. For example, in case of ERA-Interim:  [30, 366, 256, 512]. It takes ~4 hours.
# After this step, the Worldwide WT classification is ready to be loaded in the script 'WT_drivers.R'

# load daily WT classification for all years and grid points and with the same format of var365Anom:
load(paste0(workdir,"/txt/",rean.name,"/metadata.RData"))  # load lat.used and lon.used

#WTs <- array(NA, c(n.years*365, n.lat, n.lon))
WTs <- array(NA, c(n.years, 365, n.lat, n.lon))

# this loop lasts ~ 4 hours with ERA-Interim:
for(latc in lat){
    #latc <- lat[20]; lonc <- lon[1];  # for the debug

     for(lonc in lon){
          pos.latc <- which(lat == latc)
          pos.lonc <- which(lon == lonc)
                
          # Load WT classification for that point:
          WT.file <- paste0(workdir,"/txt/ERAint/all_years/WTs_",year.start,"-",year.end,"_lat_",latc,"_lon_",lonc,".txt")
    
          if(file.exists(WT.file)){
             print(paste0("pos.lat: ",pos.latc," pos.lon:",pos.lonc))
             
             WT <- read.table(WT.file,header=TRUE, stringsAsFactors=FALSE)  # load lat.used and lon.used
             
             # remove bisestile days from WT classification:
             WT$MonthDay <- paste0(WT$Month,WT$Day) # create a new column
             bis <- which(WT$MonthDay == "229")
             WT365 <- WT[-bis,]
             WT365$MonthDay <- NULL

             # insert the WT classification with 10 WTs:
             #WTs[, pos.latc, pos.lonc] <- WT$WT10.name
             for(y in year.start:year.end) WTs[y-year.start+1,, pos.latc, pos.lonc] <- WT365$WT10.num[365*(y-year.start)+1:365]     
             rm(WT365, WT, bis)
             
           }
      }
    
}

save(WTs,file=paste0(workdir,"/txt/",rean.name,"/WTs.RData"))







# only add ONE year to the existing classification:

year.start <- 1985 # first year of the old classification to update
year.end <- 2014  # last year of the old classification to update
    
y.new <- 2016  # year to add to the classification

# load daily WT classification for all years and grid points and with the same format of var365Anom:
load(paste0(workdir,"/txt/",rean.name,"/metadata.RData"))  # load lat.used and lon.used
    
for(latc in lat.used){
        #latc <- lat.used[1]; lonc <- lon.used[1];  # for the debug

        for(lonc in lon.used){
            pos.latc <- which(lat.used==latc)
            pos.lonc <- which(lon.used==lonc)
            
            #cat(paste0("Adding classification at grid point: ",(pos.latc-1)*n.lon.used + pos.lonc ,"/", n.grid.points, " "), "\r")
            print(paste0("lat: ",latc," lon: ", lonc))

            WTs <- read.table(file=paste0(workdir,"/txt/",rean.name,"/all_years/WTs_",year.start,"-",year.end,"_lat_",latc,"_lon_",lonc,".txt"),stringsAsFactors=FALSE, header=TRUE) # Load WTs
    
            WTs_new <- read.table(file=paste0(workdir,"/txt/",rean.name,"/",y.new,"/WT_year_",y.new,"_lat_",latc,"_lon_",lonc,".txt"),stringsAsFactors=FALSE, header=TRUE) # Load WTs
            
            WTs_all <- rbind(WTs, WTs_new)
            
            write.table(WTs_all, file=paste0(workdir,"/txt/",rean.name,"/all_years/WTs_",year.start,"-",year.end,"_lat_",latc,"_lon_",lonc,".txt"),row.names=FALSE, col.names=TRUE, quote=FALSE, sep=" ")
            
            rm(WTs_all)
            
        }
}

# when you have finished inserting all the new years, you have to re-rn the previous for block again, to re-create the updated WTs.RData!!!
    

    

    
} # close if on postproc
