#########################################################################################
#                                Lamb's WTs with ERA-Interim                            #
#########################################################################################
# run it from the bash with:
#
# Rscript WT_v5.R 1984
#
# being 1984 the year you want to classify the WTs;
# in this way you can run up to 8 jobs at the same time, each one producing its output files!
# You can also run it for a sequence of years with the syntax:
# 
# Rscript WT_v5.R 1980 2014
#
# and it will compute each year from 1980 to 2014, each one after finishing the previous one.
#
# If you want to run many years in parallel with just only 1 command, run from the bash:
#
# for y in {1980..2014}; do Rscript WT_v5.R &; done
#
# but it'd need 24 processors! In practice, it is possible to run only 4-8 years at time.
#
# Rscript WT_v5.R 1980 1984
# Rscript WT_v5.R 1985 1989
# Rscript WT_v5.R 1990 1994
# Rscript WT_v5.R 1995 1999
# Rscript WT_v5.R 2000 2004
# Rscript WT_v5.R 2005 2009
# Rscript WT_v5.R 2010 2014
#
# Rscript WT_v5.R 1981 1984
# Rscript WT_v5.R 1986 1989
# Rscript WT_v5.R 1991 1994
# Rscript WT_v5.R 1996 1999
#
# Rscript WT_v5.R 2001 2004
# Rscript WT_v5.R 2006 2009
# Rscript WT_v5.R 2011 2014

library(s2dverification)  # for the function Load()
library(ff)
source('/home/Earth/ncortesi/Downloads/scripts/Rfunctions.R') # for the calendar functions
#Load.path <- '/home/Earth/ncortesi/Downloads/scripts/IC3.conf'      # Load() file path
# Available reanalysis:
ERAint <- list(path = '/esnas/reconstructions/ecmwf/eraint/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc')
JRA55 <- list(path = '/esnas/reconstructions/jma/jra-55/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc')

workdir="/scratch/Earth/ncortesi/RESILIENCE/WT"    # working dir where to put the output maps and files
subdatadir="/scratch/Earth/ncortesi/RESILIENCE/WT_subdata"    # dir where to put the intermediate daily psl data (1 file for each subarray)

mslp.rean=ERAint #'ERAintDailyHighRes'  #'ERAintDailyLowRes'  # choose one of the two daily reanalysis above for loading MSLP data
mslp.rean.name <- 'ERAint'

year.start=1980     # starting year of the MSLP daily data (from the 1st of january)
year.end=2015       # ending year of the MSLP daily data (up to the 31 of December)

res=5.25 #4.9       # spacing between the Lamb grid points in the meridional direction (in the zonal direction, it is exactly the double of this value)
                    # it should be put equal to the multiple of 'psl.res' closer to 5 degrees, i.e: res = psl.res * round((5/psl.res))

low.res.size=7      # odd numb. of MSLP nearby grid points to compute the psl at each of the 16 points (both for lon and lat: the actual n.of points used is its square)

partial.end=FALSE    # put TRUE if the last year ('year.end') has not all the yearly data but stop before December the 31th; in this case, must also specify the variable below
n.days.last=334     # number of days available in the last year (used only if partial.end=TRUE) I.e: data for 2015 doesn't have December, so it has 365-31=334 days

merge=FALSE         # put TRUE if you want to concatenate all the WT classification for different years and same grid point at the end of the analysis, FALSE otherwise

##########################################################################################

args <- commandArgs(TRUE)

if(length(args) == 1) year.start <- year.end <- as.integer(args[1])
if(length(args) == 2) {year.start <- as.integer(args[1]); year.end <- as.integer(args[2])}

WTs.type<-c("NE","E","SE","S","SW","W","NW","N","C","C.NE","C.E","C.SE","C.S","C.SW","C.W","C.NW","C.N","A","A.NE","A.E","A.SE","A.S","A.SW","A.W","A.NW","A.N")
radius <- (low.res.size-1)/2  # number of grid points to use as search radius for averaging the psl at each Lamb grid point

# Load 1 day of MSLP data from the reanalysis chosen, just to detect the number of latitude and longitude points:
#MSLP <- Load('psl', NULL, mslp.rean, paste0(year.start,'0101'), storefreq = 'daily', leadtimemax = 1, output = 'lonlat', configfile = Load.path)
MSLP <- Load(var = 'psl', exp = NULL, obs = list(mslp.rean), paste0(year.start,'0101'), storefreq = 'daily', leadtimemax = 1, output = 'lonlat')

n.lat <- length(MSLP$lat)  # number of latitude values
n.lon <- length(MSLP$lon)  # number of longitude values

lon.pos <- ifelse(min(MSLP$lon>=0), TRUE, FALSE)  # set lon.pos to TRUE if the longitude values of the reanalysis grid has no negative values, FALSE if not.

# for each grid point of the reanalysis, compute the Lamb WT classification with Lamb grid centered on that point.
# we exclude only central points > +77 degrees lat and < -77 deg. because the Lamb grid needs 12.5 deg. north/south of the central point.
n.lat.unused.poles <- 20 # number of unused latitude values near each of the two poles (must exclude last 10 degrees N/S, so it depends on the spatial res. of the reanalysis)

lat.used <- MSLP$lat[-c(1:n.lat.unused.poles,(n.lat-n.lat.unused.poles):n.lat)] # latitude values used as central points 
psl.res <- diff(lat.used)[1]  # psl grid resolution
lat.used <- round(lat.used,3) # round psl values to the third decimal to save them in a file with a short file name (precision: ~100 m)
n.lat.used <- length(lat.used)
    
lon.used <- round(MSLP$lon,3) # longitude values used as central points rounded to the third decimal (precision: ~100 m)
n.lon.used <- length(lon.used)

n.grid.points <- length(lat.used)*length(lon.used)

# load all MSLP data one year at time:
for(y in year.start:year.end){
    print(paste0("Year: ",y))
    n.days <- n.days.in.a.year(y)
    if(partial.end==TRUE && y==year.end) n.days<-n.days.last # the last year can have a lower number of days

    # create two subdir where to put the yearly ascii and .ff output:
    if(!dir.exists(file.path(workdir,paste0(y,"_txt")))) dir.create(file.path(workdir,paste0(y,"_txt"))) 
    if(!dir.exists(file.path(workdir,paste0(y,"_RData")))) dir.create(file.path(workdir,paste0(y,"_RData"))) 

    #test <- 0 # dummy variable to create an ff file
    system("rm")
    test <- as.ff(0, vmode="double", file=paste0(workdir,"/WTs1_",y))  # create WTs_<year>
    #test2 <- as.ff(0, vmode="double", file=paste0(workdir,"/WTs2_",y))  # create WTs_<year>
    #test3 <- as.ff(0, vmode="double", file=paste0(workdir,"/WTs3_",y))  # create WTs_<year>


    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs1_",y), compress=FALSE)  # create WTs1_<year>.ff and WTs_<year>.RData  # there is a limit of 47000 objects that can be saved
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs2_",y), compress=FALSE)  # create WTs2_<year>.ff and WTs_<year>.RData  # with ffsave. since we have ~100000 objects to save,
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs3_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData  # we must create different output files for each year.
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs4_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData  # Furthermore, saving speed decreases with the size of
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs5_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData  # the .ff file. Thus, it's better not to save more than
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs6_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData  # ~10000 objects in one file. For this reasons, we
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs7_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData  # are creating 10 different save files.
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs8_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs9_",y), compress=FALSE)  # create WTs3_<year>.ff and WTs_<year>.RData
    ffsave(test, file=paste0(workdir,"/",y,"_RData/WTs10_",y), compress=FALSE) # create WTs3_<year>.ff and WTs_<year>.RData
    
    close(test)
    file.remove(paste0(workdir,"/WTs1_",y)) # delete the temporary ff file with test variable
    
    MSLP.year <- Load(var = 'psl', exp = NULL, obs = list(mslp.rean), paste0(y,'0101'), storefreq = 'daily', leadtimemax = n.days, output = 'lonlat')
    gc()

    # loop over the lat/lon of the central point of the Lamb classification:
    for(latc in lat.used){
        #latc <- lat.used[1]; lonc <- lon.used[1];  # for the debug

        ## first, computes the latitude coefficients:
        SF1<-1/cos(latc*pi/180) 
        ZS1<-1/(2*cos(latc*pi/180)^2)
        ZW1<-sin(latc*pi/180)/sin((latc-res)*pi/180)
        ZW2<-sin(latc*pi/180)/sin((latc+res)*pi/180)
        
        for(lonc in lon.used){
            pos.latc <- which(lat.used==latc)
            pos.lonc <- which(lon.used==lonc)

            n.point <- (pos.latc-1)*n.lon.used + pos.lonc
            cat(paste0("Grid point saved: ", n.point ,"/", n.grid.points, " "), "\r")
            
            # assign the lat and lon values of the 16 grid points of the Lamb grid centered over the central point:
            lat <- lon <- c()  # lat and lon of the 16 points of Lamb's grid
            lat[1] <- lat[2] <- latc + 2*res
            lat[3] <- lat[4] <- lat[5] <- lat[6] <- latc + res
            lat[7] <- lat[8] <- lat[9] <- lat[10] <- latc
            lat[11] <- lat[12] <- lat[13] <- lat[14] <- latc - res
            lat[15] <- lat[16] <- latc - 2*res

            lon[1] <- lon[4] <- lon[8] <- lon[12] <- lon[15] <- lonc - res
            lon[3] <- lon[7] <- lon[11] <- lonc - 3*res
            lon[2] <- lon[5] <- lon[9] <- lon[13] <- lon[16] <- lonc + res
            lon[6] <- lon[10] <- lon[14] <- lonc + 3*res

            # longitude correction for reanalysis with positive-only longitude (to stay always positive):
            if(lon.pos){
                if(lonc - res < 0){
                    lon[1] <- lon[4] <- lon[8] <- lon[12] <- lon[15] <- lonc - res + 360
                    lon[3] <- lon[7] <- lon[11] <- lonc - 3*res + 360
                }
                if(lonc - 3*res < 0 && lon - res >= 0){
                    lon[3] <- lon[7] <- lon[11] <- lonc - 3*res + 360
                }
                if(lonc + res >= 360){
                    lon[2] <- lon[5] <- lon[9] <- lon[13] <- lon[16] <- lonc + res - 360
                    lon[6] <- lon[10] <- lon[14] <- lonc + 3*res - 360
                }
                if(lonc + 3*res >= 360 && lonc + res < 360){
                    lon[6] <- lon[10] <- lon[14] <- lonc + 3*res - 360
                }
            }

            # longitude correction for reanalysis with [-180, +180] longitude (to stay always in the same interval):
            # MUST STILL UPDATE FOR CASE lonc + 15 >= 360 !!!
            if(!lon.pos){
                if(lonc - res < -180){
                    lon[1] <- lon[4] <- lon[8] <- lon[12] <- lon[15] <- lonc - res + 360
                    lon[3] <- lon[7] <- lon[11] <- lonc - 3*res + 360
                }
                if(lonc + res >= 180){
                    lon[2] <- lon[5] <- lon[9] <- lon[13] <- lon[16] <- lonc + res - 360
                    lon[6] <- lon[10] <- lon[14] <- lonc + 3*res - 360
                }
            }                
            
            # lat and lon position of the 16 Lamb grid points inside the MSLP reanalysis:
            pos.lat <- pos.lon <- c()
            for(p in 1:16){
                pos <- nearest(lat[p], lon[p], MSLP$lat, MSLP$lon)
                pos.lat[p] <- pos[1]
                pos.lon[p] <- pos[2]
            }

            #WTs <- data.frame()
            
            # mean psl at each of the 16 grid points:
            psl <- array(NA,c(n.days,16)) # for each day of the year, MSLP of the 16 points of Lamb's grid
            #for(p in 1:16) psl[,p] <- MSLP.year$obs[1,1,1,,pos.lat[p],pos.lon[p]]
            
            for(p in 1:16) {
                 
                 pos.lon.low.res <- (pos.lon[p]-radius):(pos.lon[p]+radius)
                
                 if(head(pos.lon.low.res,1) <= 0){
                     ss <- which(pos.lon.low.res <= 0)
                     pos.lon.low.res <- c(pos.lon.low.res[ss] + n.lon.used, pos.lon.low.res[-ss])
                 }
                 if(tail(pos.lon.low.res,1) > n.lon.used){
                     ss <- which(pos.lon.low.res > n.lon.used)
                     pos.lon.low.res <- c(pos.lon.low.res[-ss], pos.lon.low.res[ss]-n.lon.used)
                 }
                
                 pos.lat.low.res <- (pos.lat[p]-radius):(pos.lat[p]+radius)  # it never has to be corrected because we excluded the poles

                 MSLP.low.res <- MSLP.year$obs[1,1,1,,pos.lat.low.res,pos.lon.low.res]
                 psl[,p] <- apply(MSLP.low.res, 1, mean)
            }

            # compute geostrophical indexes (don't round them to the 3rd decimal number as done by Trigo and Martin-Vide, not to introduce an error that change 3-4% of WTs!!!):
            SF<-WF<-F<-D<-D2<-c()

            WF <- 0.5*(psl[,12]+psl[,13]) - 0.5*(psl[,4]+psl[,5])
            SF <- SF1*(0.25 * (psl[,5]+2*psl[,9]+psl[,13]) - 0.25 * (psl[,4]+2*psl[,8]+psl[,12]))
            F <- (SF*SF + WF*WF)^0.5

            if(latc < 0){
                WF <- -WF  # because in the southern hemisphere, cyclones and anticyclones spins in the opposite sense!
                SF <- -SF
            }

            # D e' la direzione del flusso; dato che l'angolo risultato di atan e' espresso in radianti, bisogna convertirlo in gradi moltiplicandolo per (180/pi)
            # D e' l'angolo tra la direzione del vento e la sua proiezione in direzione nord-sud (SF)
            D <- (180/pi) * atan(WF/SF)

            # trasforma l'angolo D iniziale in un angolo D2 sempre positivo che si calcola a partire del Nord andando in senso orario:
            Q1 <- which(WF>0 & SF>0)
            Q2 <- which(WF>0 & SF<=0)
            Q3 <- which(WF<0 & SF<=0)   # calcola quali angoli D appartengono al quadrante 1 (Q1), quali al quadrante 2, quali al 3 e quali al 4.
            Q4 <- which(WF<0 & SF>0)    # il primo quadrante e' in alto a destra, il secondo in basso a destra, il terzo in basso a sinistra, il quarto in alto a sinistra.

            # the wind direction is usually different from the angle D where the wind is blowing to, so a correction has to be made:
            D2 <- D
            D2[Q1] <- D[Q1] + 180  
            D2[Q2] <- D[Q2] + 360
            D2[Q4] <- D[Q4] + 180
            
            # individua i WTs direzionali:
            NE <- which(D2>=22.5 & D2<67.5)
            E <- which(D2>=67.5 & D2<112.5)
            SE <- which(D2>=112.5 & D2<157.5)
            S <- which(D2>=157.5 & D2<202.5)
            SW <- which(D2>=202.5 & D2<247.5)
            W <- which(D2>=247.5 & D2<292.5)
            NW <- which(D2>=292.5 & D2<337.5)
            N <- which(D2>=337.5 | D2<22.5)

            # aggiunge una colonna a destra con il WT direzionale associato all'angolo:
            D2.dir<-rep(NA,length(D2))
            D2.dir[NE]<-"NE";D2.dir[E]<-"E";D2.dir[SE]<-"SE";D2.dir[S]<-"S";D2.dir[SW]<-"SW";D2.dir[W]<-"W";D2.dir[NW]<-"NW";D2.dir[N]<-"N"

            # calcolo componenti vorticita':
            ZS<-ZW<-Z<-c()
            
            ZS <- ZS1*(0.25 * (psl[,6]+2*psl[,10]+psl[,14]) - 0.25 * (psl[,5]+2*psl[,9]+psl[,13]) - 0.25 * (psl[,4]+2*psl[,8]+psl[,12]) + 0.25 * (psl[,3]+2*psl[,7]+psl[,11]))
            ZW <- ZW1 * (0.5*(psl[,15]+psl[,16]) - 0.5*(psl[,8]+psl[,9])) - ZW2 * (0.5*(psl[,8]+psl[,9]) - 0.5*(psl[,1]+psl[,2]))
            Z <- ZS + ZW

            # Pure and hybrid WTs
            cyc.pure <- which(Z>2*F)
            anticyc.pure <- which(Z<(-2*F))
            hybrid.cyc <- which(Z>0 & abs(Z)<2*F & abs(Z)>F)
            hybrid.anticyc <- which(Z<0 & abs(Z)<2*F & abs(Z)>F)
            indeter <- which(F<6 & abs(Z)<6) # tipo di tempo indeterminato (U); the choice of 6 depend on grid size and should be changed if grid res.is higher!

            # create a data frame with all the info about the WT classification:    

            #years.period <- months.period <- days.period <- c()
            #for(y in year.start:year.end) years.period <- c(years.period, rep(y, n.days.in.a.year(y)))
            #for(y in year.start:year.end) months.period <- c(months.period, seq.months.in.a.year(y))
            #for(y in year.start:year.end) days.period <- c(days.period, seq.days.in.a.year(y))
            years.period <- rep(y, n.days)
            months.period <- seq.months.in.a.year(y)[1:n.days]
            days.period <- seq.days.in.a.year(y)[1:n.days]
            WT<-data.frame(Year=years.period, Month=months.period, Day=days.period, Direction=round(D2,3), WT.dir=D2.dir, stringsAsFactors=FALSE)

            WT$Year<-as.numeric(WT$Year);WT$Month<-as.numeric(WT$Month);WT$Day<-as.numeric(WT$Day);WT$Direction<-as.numeric(WT$Direction);WT$WT.dir<-as.character(WT$WT.dir)

            # colonna che identifica quali giorni sono di tipo ibrido e quali di tipo direzionale:
            WT$Hyb <- rep("Directional",length(D2))
            WT$Hyb[cyc.pure] <- "Pure_C"
            WT$Hyb[anticyc.pure] <- "Pure_A"
            WT$Hyb[hybrid.cyc] <- "Hybrid_C"
            WT$Hyb[hybrid.anticyc] <- "Hybrid_A"

             # colonna con la lista dei 26 WTs officiali:
            WT$WT <- WT$D2.dir
            WT$WT[cyc.pure] <- "C"
            WT$WT[anticyc.pure] <- "A"
            WT$WT[hybrid.cyc] <- paste("C.",WT$WT.dir[hybrid.cyc],sep="")
            WT$WT[hybrid.anticyc] <- paste("A.", WT$WT.dir[hybrid.anticyc],sep="")
            #WT$WT[indeter] <- "U"   # comment this line not to include the Unclassified U WT

            WT$WT.num <- match(WT$WT,WTs.type)# aggiunge una colonna con la conversione dei 26 tipi di tempo a numeri da 1 a 26

            # colonna con la lista dei 10 WTs ottenuti unendo a C e ad A gli otto WT direzionali puri;
            # i WTs ibridi si aggiungono ognuno al suo tipo direzionale corrispondente (es: C.SW si aggiunge a SW, ecc.)
            WT$WT10 <- WT$WT.dir
            WT$WT10[cyc.pure] <- "C"
            WT$WT10[anticyc.pure] <- "A"
            WT$WT10.num <- match(WT$WT10,WTs.type) # conversione dei 10 tipi di tempo a numeri da 1 a 9 + il tipo A che si converte nel numero 18

            # add the other geostrophical indexes:
            WT$SF=round(SF,5)
            WT$WF=round(WF,5)
            WT$F=round(F,5)
            WT$ZS=round(ZS,5)
            WT$ZW=round(ZW,5)
            WT$Z=round(Z,5)

            #WTs<-rbind(WTs,WT)

            # save the full data table as a .txt to exchange it with other people: 
            #write.table(WT,file=paste0(workdir,"/",y,"_txt/WTs_",mslp.rean,"_year_",y,"_lat_",latc,"_lon_",lonc,".txt"),row.names=FALSE, col.names=TRUE, quote=FALSE, sep=" ")
            output_txt <- paste0(workdir,"/",y,"_txt/WTs_",mslp.rean.name,"_year_",y,"_lat_",latc,"_lon_",lonc,".txt")
            write.table(WT,file=output_txt,row.names=FALSE, col.names=TRUE, quote=FALSE, sep=" ")
             
            # save only the classification with 10 WTs, to have a smaller (binary) file:
            #WTs<-cbind(WT$Year, WT$Month, WT$Day, WT$WT10.num)
            #WTs <- WT$WT10.num
            
            #a <- 1:1000
            #year <- 4000
            #WTs <- as.ff(a, vmode="double", file=paste0(workdir,"/WTs_",year))  # create WTs_3000
            #rm(WTs)
            
            #latc <- 70
            #lonc <- 0
            #assign(paste0("WTs_lat_",latc,"_lon_",lonc), WTs)

            # you must use this syntax when saving an Rdata with the variable name given by a string:
            #do.call(ffsave, list(paste0("WTs_lat_",latc,"_lon_", lonc), file=paste0(workdir,"/WTs_",year), compress=FALSE))  # create WTs_3000.ff and WTs_3000.RData

            #b <- 1001:2000
            #WTs2 <- as.ff(b, vmode="double", file=paste0(workdir,"/WTs_",year), overwrite=TRUE)  # create WTs_3000

            #latc <- 75
            #lonc <- 0

            #WTs <- as.ff(WT$WT10.num, vmode="double", file=paste0(workdir,"/WTs_",y), overwrite=TRUE)  # create WTs_3000
            #assign(paste0("WTs_lat_",latc,"_lon_",lonc), WTs)

            if(latc < 0) { latc.char <- paste0("n", abs(latc)) } else {latc.char <- latc}
            if(lonc < 0) { lonc.char <- paste0("n", abs(lonc)) } else {lonc.char <- lonc}
               

            # merge the two above commands in one command only, not to introduce the variable WTs:
            assign(paste0("WTs_lat_",latc.char,"_lon_",lonc.char), as.ff(WT$WT10.num, vmode="double", file=paste0(workdir,"/",y,"_RData/WTs_",y), overwrite=TRUE))  # create WTs_3000
            # you must use this syntax when saving an Rdata with the variable name given by a string:
            # (it creates WTs_3000.ff and WTs_3000.RData)

            if(n.point < 10000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs1_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 10000 & n.point < 20000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs2_",y), compress=FALSE, add=TRUE))
            if(n.point >= 20000 & n.point < 30000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs3_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 30000 & n.point < 40000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs4_",y), compress=FALSE, add=TRUE))
            if(n.point >= 40000 & n.point < 50000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs5_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 50000 & n.point < 60000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs6_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 60000 & n.point < 70000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs7_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 70000 & n.point < 80000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs8_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 80000 & n.point < 90000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs9_",y), compress=FALSE, add=TRUE)) 
            if(n.point >= 90000) do.call(ffsave, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char), file=paste0(workdir,"/",y,"_RData/WTs10_",y), compress=FALSE, add=TRUE)) 
            
            #ffsave(WTs, file=paste0(workdir,"/WTs_",year), add=TRUE)
            #save(WTs, file=paste0(workdir,"/",y,"_Rdata/10WTs_",mslp.rean.name,"_year_",y,"_lat_",latc,"_lon_",lonc,".RData"), compress=FALSE)

            rm(WT,SF,WF,F,D,ZS,ZW,W, output_txt)
            do.call(rm, list(paste0("WTs_lat_",latc.char,"_lon_", lonc.char)))
            gc()            
            
        } # close for on lonc
    } # close for on latc
    cat("\n")
    
} # close for on y 


#delete(paste0("WTs_lat_",latc,"_lon_",lonc))
#do.call(rm, list(paste0("WTs_lat_",latc,"_lon_",lonc)))
#gc()




# merge all txt data of the same lat and lon in 1 file to have all the years together and delete the yearly files:
if(merge){
    if(!dir.exists(file.path(workdir,paste0("wt_txt")))) dir.create(file.path(workdir,paste0("wt_txt")))
    
    for(latc in lat.used){
        #latc <- lat.used[1]; lonc <- lon.used[1];  # for the debug

        for(lonc in lon.used){
            pos.latc <- which(lat.used==latc)
            pos.lonc <- which(lon.used==lonc)
            cat(paste0("Merging classification at grid point: ",(pos.latc-1)*n.lon.used+pos.lonc ,"/", n.grid.points, " "), "\r")
            
            for(y in year.start:year.end){
                #print(paste0("Year: ",y))
                
                WTs <- read.table(file=paste0(workdir,"/",y,"_txt/WTs_",mslp.rean.name,"_year_",y,"_lat_",latc.char,"_lon_",lonc.char,".txt"),stringsAsFactors=FALSE, header=TRUE) # Load WTs
                if(y==year.start){
                    WTs_full_period <- WTs
                } else {
                    WTs_full_period <- rbind(WTs_full_period, WTs)
                }

            }
            
            write.table(WTs_full_period, file=paste0(workdir,"/wt_txt/WTs_",mslp.rean.name,"_",year.start,"-",year.end,"_lat_",latc.char,"_lon_",lonc.char,".txt"),row.names=FALSE, col.names=TRUE, quote=FALSE, sep=" ")
            rm(WTs_full_period)
            
        }
    }

  # delete the old yearly files in the <year>_txt directories and the directories:
  for(y in year.start:year.end){
      system(paste0("rm -fr ",workdir,"/",y,"_txt/"))    
  }
    
} # close if on merge

                                                        
