
# Creation:  28/6/2016
# Author: Nicola Cortesi
# Aim: To compute the four Euro-Atlantic weather regimes from a reanalysis or from a forecast system
# I/O: input must be in a format compatible with Load(); 1 output .RData file is created in the 'workdir' folder.
# Assumption:  its output is need by the script weather_regimes_maps.R
# Branch: weather_regimes

library(s2dverification)  # for the function Load()
library(abind)
library(Kendall)
library(reshape2)
library(TTR)
source('/home/Earth/ncortesi/Downloads/scripts/Rfunctions.R') # for the calendar functions
workdir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes"    # working dir with the input files with the WT classifications for each MSLP grid point

# available reanalysis for the geopotential (g500) or the geopotential height (z500 = g500/9.81) and for var data:
ERAint <- '/esnas/reconstructions/ecmwf/eraint/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
ERAint.name <- "ERA-Interim"

JRA55 <- '/esnas/reconstructions/jma/jra-55/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
JRA55.name <- "JRA-55"

rean <- ERAint #JRA55 #ERAint  # choose one of the two above reanalysis from where to load the input psl data

# available forecast systems for the psl and var data:
#ECMWF_S4 <- list(path = paste0('/esnas/exp/ECMWF/seasonal/0001/s004/m001/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'))
#ECMWF_S4 <- '/esnas/exp/ECMWF/seasonal/0001/s004/m001/$STORE_FREQ$_mean/psl_f6h/psl_$START_DATE$.nc'
ECMWF_S4 <- '/esnas/exp/ecmwf/system4_m1/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'
ECMWF_S4.name <- "ECMWF-S4"
member.name <- "ensemble"                       # name of the dimension with the ensemble members inside the netCDF files of S4

ECMWF_monthly <- '/esnas/exp/ECMWF/monthly/ensforhc/6hourly/$VAR_NAME$/2014$MONTH$$DAY$00/$VAR_NAME$_$START_DATE$00.nc'
ECMWF_monthly.name <- "ECMWF-Monthly"

forecast <- ECMWF_S4

fields <- rean #forecast  # put 'rean' to load pressure fields and var from reanalisis, put 'forecast' to load them from forecast systems

psl <- "psl" #"g500"                            # pressure variable to use from the chosen reanalysis
psl.name <- "SLP" # "Z500"                     # the name to show in the maps

year.start <- 1982 #1981 #1994 #1979 #1981
year.end <- 2013 #2013 #2010

res <- 0.75                # set the resolution you want to interpolate the psl and var data when loading it (i.e: set to 0.75 to use the same res of ERA-Interim)
                           # interpolation method is BILINEAR.
PCA <- FALSE               # set it to TRUE if you want to apply the PCA before the k-means cluster analysis, FALSE otherwise
variance.explained <- 0.8  # the user can set here the minimum % of variance explained by the PCs (typically 0.8 which is equal to 80%)

lat.weighting=FALSE  # set it to true to weight psl data on latitude before applying the cluster analysis
sequences=FALSE  # set it to TRUE if you want to select only days beloning to sequences of at least 5 consecutive days belonging to the same regime.

# Coordinates of the box enclosing the study region (the Northern Atlantic in this case):
lat.max <- 81 #81 #80 #70
lat.min <- 27 #30 #20 #30
lon.max <- 45 #36 #30 #40 
lon.min <- 274.5 #274.5 #270 #280 # put a positive number here because the geopotential has only positive values of longitud!

var.num <- 1                                     # Choose a variable. 1: sfcWind  2: tas

var.name <- c("sfcWind","tas")                   # name of the 'predictand' variable of the chosen reanalysis
var.name.full <- c("Wind Speed","Temperature")   # full name of the 'predictand' variable to put in the title of the graphs
var.unit <- c("m/s", "ºC")                       # unit of measure of var (for drawing color scales)

# Only for Reanalysis::
WR.period <- 1:12 #13:16  # 1-12: Jan-Dec; 13-16: Winter-Spring-Summer-Autumn  [bypassed by the optional argument of the script]

# only for Seasonal forecasts:
n.members <- 15                                  # number of members of the forecast system to load
n.leadtimes <- 216                               # number of leadtimes of the forecast system to load
startM <- 1                                      # start month (1=January, 12=December)  [bypassed by the optional arguments of the script]
leadM <- 1                                       # select only the data of one lead month:  [bypassed by the optional arguments of the script]

# Only for Monthly forecasts:
#leadtime <- 1:7 #5:11 #1                        # if fields=forecast, set the forecast time (in days) you want to compute the weather regimes.
#startdate.shift <- 1                            # if leadtime=5:11, it's better to shift all startdates of the season 1 week in the past, to fit the exact season of obs.data
                                                 # if leadtime=12:18, it's better to shift all startdates of the season 2 weeks in the past, and so on.
#forecast.year <- year.end+1                     # if fields=forecast, set the forecast year (it is usually the year after year.end)
#mes=1                                           # if fields=forecast, set the month of the first startdate of the forecast year
#day=2                                           # if fields=forecast, set the day of the first startdate of the forecast year

#############################################################################################################################
#ECMWF_monthly <- list(path = paste0('/esnas/exp/ECMWF/monthly/ensforhc/6hourly/psl/2014010200/1994_010200.nc'))
#ECMWF_monthly <- list(path = paste0('/esnas/exp/ECMWF/monthly/ensforhc/6hourly/$VAR_NAME$/2014$MONTH$$DAY$00/$VAR_NAME$_$START_DATE$00.nc'))
rean.name <- unname(ifelse(rean == ERAint, ERAint.name, JRA55.name))
forecast.name <- unname(ifelse(forecast == ECMWF_S4, ECMWF_S4.name, ECMWF_monthly.name))
fields.name <- unname(ifelse(fields == rean, rean.name, forecast.name))
if(fields.name == forecast.name) WR.period = 1
n.years <- year.end - year.start + 1

# in case the script is run with two arguments, they are assigned to the two below variables:
script.arg <- as.integer(commandArgs(TRUE))

if(length(script.arg) == 2){
  start.month <- script.arg[1]                 
  lead.month <- script.arg[2]
  WR.period <- start.month
} else {
  start.month <- startM
  lead.month <- leadM
  WR.period <- start.month
}

# in case the script is run with 1 argument, it is assumed you are using Reanalysis:
if(length(script.arg) == 1) WR.period <- script.arg[1]

var.data <- fields # dataset from which to load the input var data (reanalysis or forecasts)
var.data.name <- fields.name #ECMWF_monthly.name # ERAint.name

if(lat.min >= lat.max) stop("lat.min cannot be greater than lat.max")

days.period <- n.days.period <- period.length <- list()
for (pp in 1:17){
    days.period[[pp]] <- NA
    for(y in year.start:year.end) days.period[[pp]] <- c(days.period[[pp]], n.days.in.a.future.year(year.start, y) + pos.period(y,pp))
    days.period[[pp]] <- days.period[[pp]][-1] # remove the NA at the begin that was introduced only to be able to execute the above command
    # number of days belonging to that period from year.start to year.end:
    n.days.period[[pp]] <- length(days.period[[pp]])
    # Number of days belonging to that period in a single year: 
    period.length[[pp]] <- n.days.in.a.period(pp,1999) #+ ifelse(period==13 | period==2, 1, 0)  # using year 1999 introduce a small error in winter season length because of bisestile years
}

# Create a regular lat/lon grid to interpolate the data to the chosen res: (Notice that the regular grid is always centered at lat=0 and lon=0!)
n.lon.grid <- 360/res
n.lat.grid <- (180/res)+1
my.grid <- paste0('r',n.lon.grid,'x',n.lat.grid)
#domain<-list()
#domain$lon <- seq(0,359.999999999,res) 
#domain$lat <- c(rev(seq(0,90,res)),seq(res[1],-90,-res))

# load only 1 day of pressure data to detect the minimum and maximum lat and lon values which identify only the North Atlantic area:
if(fields.name == rean.name) domain <- Load(var = psl, exp = NULL, obs = list(list(path=fields)), '19990101', storefreq = 'daily', leadtimemax = 1, output = 'lonlat', nprocs=1)
# in the case of S4, we also interpolate it (notice that if the S4 grid has the same grid of the chosen grid (typically ERA-Interim), Load() leaves the S4 grid unchanged):
if(fields.name == ECMWF_S4.name) domain <- Load(var = "psl", exp = list(list(path=fields)), sdates=paste0(year.start,'0101'), storefreq = 'daily', dimnames=list(member=member.name), leadtimemax=1, nmember=1, output = 'lonlat', grid=my.grid, method='bilinear', nprocs=1)

#if(fields.name == ECMWF_monthly.name) domain <- Load(var = psl, exp = NULL, obs = list(list(path=fields)), paste0(year.start,'0102'), storefreq = 'daily', leadtimemax = 1, output = 'lonlat', nprocs=1)    

n.lon <- length(domain$lon)
n.lat <- length(domain$lat)

pos.lat.max <- which(lat.max - round(domain$lat,4) >= 0)[1]           # select the first latitude of the domain below the maximum latitude
pos.lat.min <- tail(which(round(domain$lat,4) - lat.min >= 0),1)      # select the last latitude of the domain over the minumum latitude
pos.lon.max <- tail(which(lon.max - round(domain$lon,4) >= 0),1)
pos.lon.min <- which(round(domain$lon,4) - lon.min >= 0)[1]

pos.lat <- if(pos.lat.min < pos.lat.max) {pos.lat.min:pos.lat.max} else {pos.lat.max:pos.lat.min}
pos.lon <- c(1:pos.lon.max,pos.lon.min:length(domain$lon)) # beware that it only consider the case where the study area cross the meridian of Greenwhich!
n.pos.lat <- length(pos.lat)
n.pos.lon <- length(pos.lon)

lat <- domain$lat[pos.lat] # lat values of chosen area only (it is smaller than the whole spatial domain loaded by the data)
lon <- domain$lon[pos.lon] # lon values of chosen area only

#lat.min.area <- domain$lat[pos.lat.min] # min and max lat/lon of the chosen area only (same as lat.max, but its values correspond to actual values in the lat vector)
#lat.max.area <- domain$lat[pos.lat.max]
#lon.min.area <- domain$lon[pos.lon.min]
#lon.max.area <- domain$lon[pos.lon.max]

#rm(domain)

# Load psl data:
if(fields.name == rean.name){ # Load daily psl data in the reanalysis case:
    psleuFull <-array(NA,c(n.days.in.a.yearly.period(year.start,year.end), n.pos.lat, n.pos.lon))

    for (y in year.start:year.end){
        var <- Load(var = psl, exp = NULL, obs = list(list(path=fields)), paste0(y,'0101'), storefreq = 'daily', leadtimemax = n.days.in.a.year(y), output = 'lonlat',
                    latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max)
        psleuFull[seq.days.in.a.future.year(year.start, y),,] <- var$obs
        rm(var)
        gc()
    }
}

if(fields.name == ECMWF_S4.name){ # in this case, we can load only the data for 1 month at time (since each month needs ~3 GB of memory)    
    if(start.month <= 9) {start.month.char <- paste0("0",as.character(start.month))} else {start.month.char <- start.month}

    psleuFull <- Load(var = psl, exp = list(list(path=fields)), obs = NULL, sdates=paste0(year.start:year.end, start.month.char,'01'), dimnames=list(member=member.name), nmember=n.members, leadtimemax=n.leadtimes, storefreq='daily', output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max, nprocs=1)$mod #, grid=my.grid, method='bilinear')

    #Load(var = "tas", exp = list(list(path="/esnas/exp/ecmwf/system4_m1/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc")), obs = NULL, sdates='19820101', dimnames=list(member="ensemble"), nmember=1, leadtimemax=1, storefreq='daily', output = 'lonlat', grid="r480x241", lonmin=20)

}

if(fields.name == ECMWF_monthly.name){
    # Load 6-hourly psl data in the forecast case:    
    psleuFull <-array(NA, c(n.sdates*n.years, n.leadtimes, n.pos.lat, n.pos.lon))  # daily order: 19940102 19950102 ... 20130102 19940109 19950109 ... 20130109 ...
    n.leadtimes <- length(leadtime)
    sdates <- weekly.seq(forecast.year,mes,day)
    n.sdates <- length(sdates)
    
    for (startdate in 1:n.sdates){
       for(lday in leadtime){
          var <- Load(var = psl, exp = NULL, obs = list(list(path=fields)), paste0(year.start:year.end, substr(sdates[startdate],5,6), substr(sdates[startdate],7,8) ), storefreq = 'daily', leadtimemin = lday*4-3, leadtimemax = lday*4, output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max)
          
          mean.lday <- apply(var$obs, c(3,5,6), mean, na.rm=T)
          rm(var)
          gc()
                           
          psleuFull[(1+(startdate-1)*n.years):(startdate*n.years), lday-leadtime[1]+1,,] <- mean.lday
          rm(mean.lday)
          gc()
       }
    }
}    

if(psl=="g500") psleuFull <- psleuFull/9.81   # convert geopotential to geopotential height (in m)
if(psl=="psl") psleuFull <- psleuFull/100   # convert MSLP in Pascal to MSLP in hPa
gc()

#save.image(file=paste0(workdir,"/weather_regimes_",fields.name,"_",year.start,"-",year.end,".RData"))
#load(file=paste0(workdir,"/weather_regimes_",fields.name,"_",year.start,"-",year.end,".RData"))

# compute the cluster analysis:
my.PCA <- list()
my.cluster <- list()
my.cluster.array <- list()
tot.variance <- list()
n.pcs <- my.cluster <- c()
    
#WR.period=1 # for the debug   
for(p in WR.period){
    # Select only the data of the month/season we want:
    if(fields.name == rean.name){
        pslPeriod <- psleuFull[days.period[[p]],,] # select only days in the chosen period (i.e: winter)
            
        # weight the pressure fields based on latitude (apply it to anomalies, not to absolute values!):
        if(lat.weighting){
            lat.weighted <- cos(lat*pi/180)^0.5
            lat.weighted.array <- InsertDim(InsertDim(lat.weighted,2,n.pos.lon), 1, n.days.period[[p]])
            psl.kmeans <- pslPeriod * lat.weighted.array
            rm(lat.weighted.array)
            gc()
        }

        # select period data from psleuFull to use it as input of the cluster analysis:    
        psl.kmeans <- pslPeriod
        dim(psl.kmeans) <- c(n.days.period[[p]], n.pos.lat*n.pos.lon) # convert array in a matrix!
        rm(pslPeriod, pslPeriod.weighted)
    }

    # in this case of S4 we don't need to select anything because we already loaded only 1 month of data!
    if(fields.name == ECMWF_S4.name){
        
        ## climatology plots:
        
        ## load eraint data:
        ## ERAint <- '/esnas/reconstructions/ecmwf/eraint/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
        ## my.years <- year.start:year.end
        ## var <- Load(var = "sfcWind", exp = NULL, obs = list(list(path=ERAint)), paste0(my.years,'0101'), storefreq = 'daily', leadtimemax = 216, output = 'lonlat',latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max) # -273.16 for tas # or /100 for psl
        
        ## # check S4 lon and lat:
        ## #varS4 <- Load(var = psl, exp = list(list(path=fields)), obs = NULL, sdates=paste0(year.start, start.month.char,'01'), dimnames=list(member=member.name), nmember=n.members, leadtimemax=1, storefreq='daily', output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max) #, grid=my.grid, method='bilinear')
        
        ## # draw climatologies:
        ## drift <- apply(psleuFull[,,,,1,1,drop=F], c(1,2,4), mean, na.rm=T)
        ## #matplot(t(drift[1,,]),type="l", col=1:15, lty=2, pch=19, cex=.5)
        ## #matplot(t(drift[1,,1:31]),type="b", col=1:15, lty=1, pch=19, cex=.5) # zoom over the first leadtime
        
        ## matplot(t(drift[1,,]),type="l", col="gray60", lty=3, pch=19, cex=.1) #, ylim=c(-25,5))
        
        ## true.climate <- apply(psleuFull[,,,,1,1,drop=F], c(1,4), mean, na.rm=T)
        ## lines(true.climate[1,],type="l", col="red", lty=1, pch=19, lwd=2)

        ## true.climate.5d <- stats::filter(true.climate[1,], rep(1/5,5), sides=2)
        ## lines(true.climate.5d,type="l", col="orange", lty=1, pch=19, lwd=2)

        ## s4.data <- data.frame(s4=true.climate[1,], day=1:216)
        ## s4.loess <- loess(s4 ~ day, s4.data, span=0.50)
        ## s4.pred <- predict(s4.loess)
        ## lines(s4.pred,type="l", col="purple", lty=1, pch=19, lwd=2)

        ## col.monthly <- "brown"
        ## true.climate1 <- apply(psleuFull[,,,1:31,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(rep(true.climate1,31),type="l", col=col.monthly, lty=1, pch=19, lwd=2)
        ## true.climate2 <- apply(psleuFull[,,,32:59,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(c(rep(NA,31),rep(true.climate2,28)),type="l", col=col.monthly, lty=1, pch=19, lwd=2)
        ## true.climate3 <- apply(psleuFull[,,,60:90,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(c(rep(NA,59),rep(true.climate3,31)),type="l", col=col.monthly, lty=1, pch=19, lwd=2)
        ## true.climate4 <- apply(psleuFull[,,,91:120,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(c(rep(NA,90),rep(true.climate4,30)),type="l", col=col.monthly, lty=1, pch=19, lwd=2)
        ## true.climate5 <- apply(psleuFull[,,,121:151,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(c(rep(NA,120),rep(true.climate5,31)),type="l", col=col.monthly, lty=1, pch=19, lwd=2)
        ## true.climate6 <- apply(psleuFull[,,,151:180,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(c(rep(NA,150),rep(true.climate6,30)),type="l", col=col.monthly, lty=1, pch=19, lwd=2)
        ## true.climate7 <- apply(psleuFull[,,,181:211,1,1,drop=F], 1, mean, na.rm=T)
        ## lines(c(rep(NA,180),rep(true.climate7,31)),type="l", col=col.monthly, lty=1, pch=19, lwd=2)

        
        ## era <- apply(var$obs[,,,,1,1,drop=F], c(1,2,4), mean, na.rm=T)
        ## plot(era[1,1,1:216],type="l", col="black", lty=1, pch=19, lwd=2)

        ## era.5d <-  stats::filter(era[1,1,1:216], rep(1/5,5), sides=2)
        ## lines(era.5d,type="l", col="darkgreen", lty=1, pch=19, lwd=2)

        ## era.data <- data.frame(era=era[1,1,1:216], day=1:216)
        ## era.loess <- loess(era ~ day, era.data, span=0.50)
        ## era.pred <- predict(era.loess)
        ## lines(era.pred,type="l", col="blue", lty=1, pch=19, lwd=2)

        ## era.data2 <- data.frame(era=era[1,1,1:216], day=1:216)
        ## era.loess2 <- loess(era ~ day, era.data2, span=0.3)
        ## era.pred2 <- predict(era.loess2)
        ## lines(era.pred2,type="l", col="purple", lty=1, pch=19, lwd=2)

        ## era.data3 <- data.frame(era=era[1,1,1:216], day=1:216)
        ## era.loess3 <- loess(era ~ day, era.data3, span=0.35)
        ## era.pred3 <- predict(era.loess3)
        
        ## col.monthly.erai <- "turquoise3"
        ## era1 <- apply(var$obs[,,,1:31,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(rep(era1,31),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)
        ## era2 <- apply(var$obs[,,,32:59,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(c(rep(NA,31),rep(era2,28)),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)
        ## era3 <- apply(var$obs[,,,60:90,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(c(rep(NA,59),rep(era3,31)),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)
        ## era4 <- apply(var$obs[,,,91:120,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(c(rep(NA,90),rep(era4,30)),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)
        ## era5 <- apply(var$obs[,,,121:151,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(c(rep(NA,120),rep(era5,31)),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)
        ## era6 <- apply(var$obs[,,,151:180,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(c(rep(NA,150),rep(era6,30)),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)
        ## era7 <- apply(var$obs[,,,181:211,1,1,drop=F], c(1,2), mean, na.rm=T)
        ## lines(c(rep(NA,180),rep(era7,30)),type="l", col=col.monthly.erai, lty=1, pch=19, lwd=2)

        
        ## # anomalias:
        ## point.type <- "l"
        ## true.climate.anom <- psleuFull[1,1,pos.last.year,,1,1] - true.climate[1,]
        ## plot(true.climate.anom,type=point.type, col="red", lty=1, pch=19, lwd=2, cex=.3)
        ## lines(rep(0,216))

        ## true.climate.5d.anom <-  psleuFull[1,1,pos.last.year,,1,1] - true.climate.5d
        ## lines(true.climate.5d.anom,type=point.type, col="orange", lty=1, pch=19, lwd=2, cex=.3)

        ## s4.pred.anom <-  psleuFull[1,1,pos.last.year,,1,1] -  s4.pred   
        ## lines(s4.pred.anom,type=point.type, col="purple", lty=1, pch=19, lwd=2, cex=.3)

        ## s4.monthly.clim <- c(rep(true.climate1,31),rep(true.climate2,28),rep(true.climate3,31),rep(true.climate4,30),rep(true.climate5,31),rep(true.climate6,30),rep(true.climate7,31), rep(NA,4))
        ## s4.monthly.anom <- psleuFull[1,1,pos.last.year,,1,1] - s4.monthly.clim
        ## lines(s4.monthly.anom,type=point.type, col="brown", lty=1, pch=19, lwd=2, cex=.3)

        
        ## pos.last.year <- dim(var$obs)[3]
        ## era.anom <- var$obs[1,1,pos.last.year,1:216,1,1] - era[1,1,1:216]
        ## plot(era.anom,type=point.type, col="black", lty=1, pch=19, lwd=2)
        ## lines(rep(0,216))

        ## era.anom.5d <- var$obs[1,1,pos.last.year,1:216,1,1] - era.5d[1:216]
        ## lines(era.anom.5d,type=point.type, col="darkgreen", lty=1, pch=19, lwd=2)

        ## era.anom.pred <- var$obs[1,1,pos.last.year,1:216,1,1] - era.pred[1:216]
        ## lines(era.anom.pred,type=point.type, col="blue", lty=1, pch=19, lwd=2)

        ## era.anom.pred2 <- var$obs[1,1,pos.last.year,1:216,1,1] - era.pred2[1:216]
        ## era.anom.pred3 <- var$obs[1,1,pos.last.year,1:216,1,1] - era.pred3[1:216]

        ## era.monthly.clim <- c(rep(era1,31),rep(era2,28),rep(era3,31),rep(era4,30),rep(era5,31),rep(era6,30),rep(era7,31), rep(NA,4))
        ## era.anom.monthly <- var$obs[1,1,pos.last.year,1:216,1,1] - era.monthly.clim
        ## lines(era.anom.monthly,type=point.type, col="turquoise3", lty=1, pch=19, lwd=2)

        ## # Nube plot:
        ## plot(era.anom.pred,type="l", col="blue", lty=1, pch=19, lwd=2)
        ## grid(nx=10,lwd=2)
        ## lines(era.anom.pred2,type="l", col="purple", lty=1, pch=19, lwd=2)
        ## lines(era.anom.pred3,type="l", col="turquoise3", lty=1, pch=19, lwd=2)
        ## lines(era.anom.5d,type="l", col="darkgreen", lty=1, pch=19, lwd=2)

        ## lines(era.5d,type="l", col="darkgreen", lty=1, pch=19, lwd=2)
        ## lines(era.pred,type="l", col="blue", lty=1, pch=19, lwd=2)
        ## lines(rep(0,216))

        ## lines(era.pred2,type="l", col="purple", lty=1, pch=19, lwd=2)
        ## lines(era.pred3,type="l", col="", lty=1, pch=19, lwd=2)

        
        ## # fit alfa parameter for each grid point:
        ## i=1
        ## j=1
        ## era <- apply(var$obs[,,,,i,j,drop=F], c(1,2,4), mean, na.rm=T)
        ## era.data <- data.frame(era=era[1,1,1:216], day=1:216)
        ## k=0; error <- c()
        ## for (alfa in seq(0.25,0.50,0.01)){
        ##   k=k+1
        ##   era.loess <- loess(era ~ day, era.data, span=alfa)
        ##   era.pred <- predict(era.loess)
        ##   error[k] <- mean(abs(era.pred - era[1,1,1:216]))
        ## }
        
        # convert psl in daily anomalies with the LOESS filter:
        pslPeriodClim <- apply(psleuFull, c(1,4,5,6), mean, na.rm=T)
        pslPeriodClimLoess <- pslPeriodClim
        for(i in n.pos.lat){
            for(j in n.pos.lon){
                 my.data <- data.frame(ens.mean=pslPeriodClim[1,,i,j], day=1:n.leadtimes)
                 my.loess <- loess(ens.mean ~ day, my.data, span=0.35)
                 pslPeriodClimLoess[1,,i,j] <- predict(my.loess)
            }
        }

        pslPeriodClim2 <- InsertDim(InsertDim(pslPeriodClimLoess, 2, n.years), 2, n.members)
        
        ## s4.data <- data.frame(s4=pslPeriodClim[1,], day=1:216)
        ## s4.loess <- loess(s4 ~ day, s4.data, span=0.35)
        ## s4.pred <- predict(s4.loess)
        rm(pslPeriodClim)
        gc()

        psleuFull <- psleuFull - pslPeriodClim2    
        rm(pslPeriodClim2)
        gc()
        
        ## # convert psl in daily anomalies computed with the 10-days leadtime window including the 9 days AFTER each day:
        ## n.leadtimes <- dim(psleuFull)[4]
        ## pslPeriodClim10 <- array(NA, c(1, n.members, n.years, n.leadtimes-9,n.pos.lat,n.pos.lon))
        ## for(year in 1:n.years){
        ##     for(lead in 1:(n.leadtimes-9)){
        ##         pslPeriodClim10[1,,year,lead,,] <- (psleuFull[1,,year,lead,,] + psleuFull[1,,year,lead+1,,] + psleuFull[1,,year,lead+2,,] + psleuFull[1,,year,lead+3,,] + psleuFull[1,,year,lead+4,,] + psleuFull[1,,year,lead+5,,] + psleuFull[1,,year,lead+6,,] + psleuFull[1,,year,lead+7,,] + psleuFull[1,,year,lead+8,,] + psleuFull[1,,year,lead+9,,])/10
        ##     }
        ## }

        ## pslPeriodClim10mean <- apply(pslPeriodClim10, c(1,4,5,6), mean, na.rm=T)
        ## pslPeriodClim10mean2 <- InsertDim(InsertDim(pslPeriodClim10mean,2,n.years),2,n.members)
        
        ## psleuFull10 <- psleuFull[1,,,1:(n.leadtimes-9),,,drop=F] -  pslPeriodClim10mean2
        
        ## psleuFull <- psleuFull10
        ## rm(pslPeriodClim2, psleuFull10, pslPeriodClim10, pslPeriodClim10mean2, psleuFull10mean, pslPeriodClim10mean)
        ## gc()

        # select data only for the startmonth and leadmonth to study:
        chosen.month <- start.month + lead.month   # find which month we want to select data
        if(chosen.month > 12) chosen.month <- chosen.month - 12
        
        for(y in year.start:year.end){
           leadtime.min <- 1 + n.days.in.a.monthly.period(start.month, chosen.month, y) - n.days.in.a.month(chosen.month, y)
           leadtime.max <- leadtime.min + n.days.in.a.month(chosen.month, y) - 1
           if (n.days.in.a.month(chosen.month, y) == 29) leadtime.max <- leadtime.max - 1  # remove the 29th of February to have the same n. of elements for all years
           int.leadtimes <- leadtime.min:leadtime.max
           n.leadtimes <- leadtime.max - leadtime.min + 1

           if(y == year.start) {
               pslPeriod <- psleuFull[,,1,int.leadtimes,,,drop=FALSE]
           } else {
               pslPeriod <- unname(abind(pslPeriod, psleuFull[,,y-year.start+1,int.leadtimes,,,drop=FALSE], along=3))
           }
        }


        ## # if you didn't convert psl data in anomalies until now, you can convert psl data in monthly anomalies:
        ## pslPeriodClim <- apply(pslPeriod, c(1,5,6), mean, na.rm=T)
        ## pslPeriodClim2 <- InsertDim(InsertDim(InsertDim(pslPeriodClim,2,n.leadtimes), 2, n.years), 2, n.members)

        ## pslPeriod <- pslPeriod - pslPeriodClim2    
        ## rm(pslPeriodClim, pslPeriodClim2)
        ## gc()
        
        # note that the data order in psl.kmeans[,X] is: year1day1, year1day2, ... , year1day31, year2day1, year2day2, ... , year2day28
        psl.melted <- melt(pslPeriod[1,,,,,, drop=FALSE], varnames=c("Exp","Member","StartYear","LeadDay","Lat","Lon")) 
        psl.kmeans <- unname(acast(psl.melted, Member + StartYear + LeadDay ~ Lat + Lon))
        
        #rm(pslPeriod)
        gc()        
    }

    if(fields.name == ECMWF_monthly.name){
        my.startdates.period <- months.period(forecast.year,mes,day,p) # get which startdates belong to the chosen period

        days.period <- list() # get which days inside psleuFull belong to the chosen period
        for(startdate in my.startdates.period){
           days.period[[p]] <- c(days.period[[p]], (1+(startdate-1)*n.years):(startdate*n.years))
        }
        
        # in winter you must remove the 2nd startdate (20140109) because its data is bad, as you can see with: plot(psl.kmeans[,1:2])
        # if(fields.name == forecast.name && period == 13) psl.kmeans[21:40,] <- NA
    }

        
    # compute the PCs or not:
    if(PCA){ 
       my.seq <- seq(1, n.pos.lat*n.pos.lon, 9) # select only 1 point of 9
       pslcut <- psl.kmeans[,my.seq]
  
       my.PCA[[p]] <- princomp(pslcut,cor=FALSE)
       tot.variance[[p]] <- head(cumsum(my.PCA[[p]]$sdev^2/sum(my.PCA[[p]]$sdev^2)),50)  # check how many PCAs to keep basing on the sum of their expl. variance
       n.pcs[p] <- head(as.numeric(which(tot.variance[[p]] > variance.explained)),1) # select only the pcs that explains at least 80% of variance
       my.cluster[[p]] <- kmeans(my.PCA[[p]]$scores[,1:n.pcs[p]], centers=4, iter.max=100, nstart=30)  # 4 is the number of clusters
       rm(pslcut)
    } else {  # 4 is the number of clusters: 
       if(fields.name == rean.name) my.cluster[[p]] <- kmeans(psl.kmeans[which(!is.na(psl.kmeans[,1])),], centers=4, iter.max=100, nstart=30)
       if(fields.name == ECMWF_S4.name) {
           my.cluster[[p]] <- kmeans(psl.kmeans, centers=4, iter.max=100, nstart=30, trace=TRUE)   
           my.cluster.array[[p]] <- array(my.cluster[[p]]$cluster, c(n.leadtimes, n.years, n.members)) # in reverse order compared to the definition of psl.kmeans
            
           #plot(SMA(my.cluster[[p]]$centers[1,],201),type="l",col="gold",ylim=c(-20,20));lines(SMA(my.cluster[[p]]$centers[2,],201),type="l",col="red"); lines(SMA(my.cluster[[p]]$centers[3,],201),type="l",col="green");lines(SMA(my.cluster[[p]]$centers[4,],201),type="l",col="blue");lines(SMA(psl.kmeans[29,],201),type="l",col="gray");lines(rep(0,14410),type="l",col="black",lty=2)
       }
    }
    
    # save the time series output of the cluster analysis:
    save(my.cluster, my.cluster.array, my.PCA, tot.variance, n.pcs, file=paste0(workdir,"/",fields.name,"_",var.name[var.num],"_",my.period[p],"_ClusterData.RData"))
    rm(psl.kmeans)
    gc()
    

#save.image(file=paste0(workdir,"/weather_regimes_",fields.name,"_",year.start,"-",year.end,".RData"))
#load(file=paste0(workdir,"/weather_regimes_",fields.name,"_",year.start,"-",year.end,".RData"))

# Load var data:
if(fields.name == rean.name){ # Load daily var data in the reanalysis case:
    vareuFull <-array(NA,c(n.days.in.a.yearly.period(year.start,year.end), n.pos.lat, n.pos.lon))

    for (y in year.start:year.end){
        var <- Load(var = var.name[var.num], exp = NULL, obs = list(var.data), paste0(y,'0101'), storefreq = 'daily', leadtimemax = n.days.in.a.year(y), output = 'lonlat',
                    latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max)
        vareuFull[seq.days.in.a.future.year(year.start, y),,] <- var$obs
        rm(var)
        gc()
    }
}
    
## if(fields.name == ECMWF_S4.name) {    
##      if(start.month <= 9) {start.month.char <- paste0("0",as.character(start.month))} else {start.month.char <- start.month}

##      my.years <- year.start:year.end
##      vareuFull <- Load(var = var.name[var.num], exp = list(list(path=fields)), obs = NULL, sdates=paste0(my.years, start.month.char,'01'), dimnames=list(member=member.name), nmember=n.members, leadtimemax=216, storefreq='daily', output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max)$mod #, grid=my.grid, method='bilinear')
## }

if(fields.name == ECMWF_monthly.name){  # Load 6-hourly var data in the monthly forecast case:    
    sdates <- weekly.seq(forecast.year,mes,day)
    vareuFull <-array(NA,c(length(sdates)*(year.end-year.start+1), n.pos.lat, n.pos.lon))
    
    for (startdate in 1:length(sdates)){
       var <- Load(var = var.name[var.num], exp = NULL, obs = list(var.data), paste0(year.start:year.end, substr(sdates[startdate],5,6), substr(sdates[startdate],7,8) ), storefreq = 'daily', leadtimemin=leadtime*4-3, leadtimemax=leadtime*4, output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max)
       temp <- apply(var$obs, c(1,3,5,6), mean, na.rm=T)
       vareuFull[(1+(startdate-1)*(year.end-year.start+1)):(startdate*(year.end-year.start+1)),,] <- temp
       rm(temp,var)
       gc()
    }
    
}

#my.grid<-paste0('r',n.lon,'x',n.lat)
#coord <- Load(var = 'sfcWind', exp = list(ECMWF_monthly), obs = NULL, sdates=paste0(2013,'0102'), leadtimemin = 1, leadtimemax=1,  output = 'lonlat', nprocs=1, grid=my.grid, method='bilinear')

#save.image(file=paste0(workdir,"/weather_regimes_",fields.name,"_",var.name[var.num],"_",year.start,"-",year.end,".RData"))
#load(file=paste0(workdir,"/weather_regimes_",fields.name,"_",var.name[var.num],"_",year.start,"-",year.end,".RData"))
    
for(p in WR.period){ # 1-12: Jan-Dec; 13-16: Winter-Spring-Summer-Autumn; 17: Year  
    
    if(fields.name == rean.name){
      cluster.sequence <- my.cluster[[p]]$cluster

      if(sequences){ # it works only for rean data!!!
          # for each of the 4 clusters, remove the days not belonging to sequences of at least 5 days:
          # warning: first 4 days and last 4 days are not take into account y the algorithm and are treated as not belonging to any sequence (sequ=FALSE)    
          wrdiff <- diff(my.cluster[[p]]$cluster)

          sequ <- rep(FALSE, n.days.period[[p]])
          for(i in 5:(n.days.period[[p]]-5)){
              sequ[i] <- TRUE
              if(wrdiff[i] != 0 & wrdiff[i+1] != 0) sequ[i] = FALSE
              if(wrdiff[i] != 0 & wrdiff[i+2] != 0) sequ[i] = FALSE
              if(wrdiff[i] != 0 & wrdiff[i+3] != 0) sequ[i] = FALSE
              if(wrdiff[i] != 0 & wrdiff[i+4] != 0) sequ[i] = FALSE
              if(wrdiff[i-1] != 0 & wrdiff[i] != 0) sequ[i] = FALSE
              if(wrdiff[i-1] != 0 & wrdiff[i+1] != 0) sequ[i] = FALSE
              if(wrdiff[i-1] != 0 & wrdiff[i+2] != 0) sequ[i] = FALSE
              if(wrdiff[i-1] != 0 & wrdiff[i+3] != 0) sequ[i] = FALSE
              if(wrdiff[i-2] != 0 & wrdiff[i] != 0) sequ[i] = FALSE
              if(wrdiff[i-2] != 0 & wrdiff[i+1] != 0) sequ[i] = FALSE
              if(wrdiff[i-2] != 0 & wrdiff[i+2] != 0) sequ[i] = FALSE
              if(wrdiff[i-3] != 0 & wrdiff[i] != 0) sequ[i] = FALSE
              if(wrdiff[i-3] != 0 & wrdiff[i+1] != 0) sequ[i] = FALSE
              if(wrdiff[i-4] != 0 & wrdiff[i] != 0) sequ[i] = FALSE
          } # close for loop on i
          
          # sequence of daily cluster numbers without the days that doesn't belong to sequences of 5 or more days:
          cluster.sequence <- my.cluster[[p]]$cluster * sequ
          rm(wrdiff, sequ)
      }
      
      # Replace the days belonging to a regime with the days belonging to a sequence of at least 5 days of that regime:
      wr1 <- which(cluster.sequence == 1)
      wr2 <- which(cluster.sequence == 2)
      wr3 <- which(cluster.sequence == 3)
      wr4 <- which(cluster.sequence == 4)
      
      # yearly frequency of each regime:    
      wr1y <- wr2y <- wr3y <-wr4y <- c()
      for(y in year.start:year.end){
        # for both reanalysis and S4, the time order is always: year1day1, year1day2, ..., year1day31, year2day1, year2day2, ..., year2day28, etc,
        wr1y[y-year.start+1] <- length(which(cluster.sequence[(y-year.start)*period.length[[p]]+(1:period.length[[p]])] == 1))
        wr2y[y-year.start+1] <- length(which(cluster.sequence[(y-year.start)*period.length[[p]]+(1:period.length[[p]])] == 2))
        wr3y[y-year.start+1] <- length(which(cluster.sequence[(y-year.start)*period.length[[p]]+(1:period.length[[p]])] == 3))
        wr4y[y-year.start+1] <- length(which(cluster.sequence[(y-year.start)*period.length[[p]]+(1:period.length[[p]])] == 4))
      }

      # convert to frequencies in %:
      wr1y <- wr1y/period.length[[p]]
      wr2y <- wr2y/period.length[[p]]
      wr3y <- wr3y/period.length[[p]]
      wr4y <- wr4y/period.length[[p]]
          
      pslPeriod <- psleuFull[days.period[[p]],,]
      pslPeriodClim <- apply(pslPeriod, c(2,3), mean, na.rm=T)
      pslPeriodClim2 <- InsertDim(pslPeriodClim, 1, n.days.period[[p]])
      pslPeriodAnom <- pslPeriod - pslPeriodClim2
        
      rm(pslPeriod, pslPeriodClim, pslPeriodClim2)
      gc()
    
      pslwr1 <- pslPeriodAnom[wr1,,]
      pslwr2 <- pslPeriodAnom[wr2,,]
      pslwr3 <- pslPeriodAnom[wr3,,]
      pslwr4 <- pslPeriodAnom[wr4,,]
          
      rm(pslPeriodAnom)
      gc()
        
      pslwr1mean <- apply(pslwr1,c(2,3),mean,na.rm=T)
      pslwr2mean <- apply(pslwr2,c(2,3),mean,na.rm=T)
      pslwr3mean <- apply(pslwr3,c(2,3),mean,na.rm=T)
      pslwr4mean <- apply(pslwr4,c(2,3),mean,na.rm=T)
      rm(pslwr1,pslwr2,pslwr3,pslwr4)

      # in the reanalysis case, we also measure the impact maps:        

       # similar selection of above, but for var instead of psl:
       varPeriod <- vareuFull[days.period[[p]],,] # select only var data during the chosen period

       varPeriodClim <- apply(varPeriod, c(2,3), mean, na.rm=T)
       varPeriodClim2 <- InsertDim(varPeriodClim, 1, n.days.period[[p]])
       varPeriodAnom <- varPeriod - varPeriodClim2
       rm(varPeriod, varPeriodClim, varPeriodClim2)
       gc()
    
       #PlotEquiMap2(varPeriodClim[,EU], lon[EU], lat, filled.continents = FALSE, cols=my.cols.var, intxlon=10, intylat=10, cex.lab=1.5) # plot the climatology of the period    
       varPeriodAnom1 <- varPeriodAnom[wr1,,]
       varPeriodAnom2 <- varPeriodAnom[wr2,,]
       varPeriodAnom3 <- varPeriodAnom[wr3,,]
       varPeriodAnom4 <- varPeriodAnom[wr4,,]

       varPeriodAnom1mean <- apply(varPeriodAnom1,c(2,3),mean,na.rm=T)
       varPeriodAnom2mean <- apply(varPeriodAnom2,c(2,3),mean,na.rm=T)
       varPeriodAnom3mean <- apply(varPeriodAnom3,c(2,3),mean,na.rm=T)
       varPeriodAnom4mean <- apply(varPeriodAnom4,c(2,3),mean,na.rm=T)
    
       varPeriodAnomBoth1 <- abind(varPeriodAnom, varPeriodAnom1, along = 1)
       pvalue1 <- apply(varPeriodAnomBoth1, c(2,3), function(x) t.test(x[1:n.days.period[[p]]],x[(n.days.period[[p]]+1):length(x)])$p.value)
       rm(varPeriodAnomBoth1, varPeriodAnom1)
       gc()
    
       varPeriodAnomBoth2 <- abind(varPeriodAnom, varPeriodAnom2, along = 1)
       pvalue2 <- apply(varPeriodAnomBoth2, c(2,3), function(x) t.test(x[1:n.days.period[[p]]],x[(n.days.period[[p]]+1):length(x)])$p.value)
       rm(varPeriodAnomBoth2, varPeriodAnom2)
       gc()
    
       varPeriodAnomBoth3 <- abind(varPeriodAnom, varPeriodAnom3, along = 1)
       pvalue3 <- apply(varPeriodAnomBoth3, c(2,3), function(x) t.test(x[1:n.days.period[[p]]],x[(n.days.period[[p]]+1):length(x)])$p.value)
       rm(varPeriodAnomBoth3, varPeriodAnom3)
       gc()
    
       varPeriodAnomBoth4 <- abind(varPeriodAnom, varPeriodAnom4, along = 1)
       pvalue4 <- apply(varPeriodAnomBoth4, c(2,3), function(x) t.test(x[1:n.days.period[[p]]],x[(n.days.period[[p]]+1):length(x)])$p.value)
       rm(varPeriodAnomBoth4, varPeriodAnom4)
    
       rm(varPeriodAnom)
       gc()
    
       # save all the data necessary to redraw the graphs when we know the right regime:
       save(lon, lon.max, lat, psl, psl.name, my.period, p, workdir,rean.name,fields.name,var.num,var.name,var.name.full,var.unit,year.start,year.end,WR.period,pslwr1mean,pslwr2mean,pslwr3mean,pslwr4mean, varPeriodAnom1mean, varPeriodAnom2mean, varPeriodAnom3mean,varPeriodAnom4mean,wr1y,wr2y,wr3y,wr4y,pvalue1,pvalue2,pvalue3,pvalue4,cluster.sequence,file=paste0(workdir,"/",fields.name,"_",var.name[var.num],"_",my.period[p],"_mapdata.RData"))
    
       rm(pvalue1,pvalue2,pvalue3,pvalue4)
       rm(pslwr1mean,pslwr2mean,pslwr3mean,pslwr4mean)
       rm(varPeriodAnom1mean,varPeriodAnom2mean,varPeriodAnom3mean,varPeriodAnom4mean)
       gc()    
      
    }
        
    if(fields.name == ECMWF_S4.name){
      cluster.sequence <- my.cluster[[p]]$cluster #as.vector(my.cluster.array[[p]]) 

      wr1 <- which(cluster.sequence == 1)
      wr2 <- which(cluster.sequence == 2)
      wr3 <- which(cluster.sequence == 3)
      wr4 <- which(cluster.sequence == 4)

      wr1y <- apply(my.cluster.array[[p]] == 1, 2, sum)
      wr2y <- apply(my.cluster.array[[p]] == 2, 2, sum)
      wr3y <- apply(my.cluster.array[[p]] == 3, 2, sum)
      wr4y <- apply(my.cluster.array[[p]] == 4, 2, sum)

      # convert to frequencies in %:
      wr1y <- wr1y/(n.leadtimes*n.members)
      wr2y <- wr2y/(n.leadtimes*n.members)
      wr3y <- wr3y/(n.leadtimes*n.members)
      wr4y <- wr4y/(n.leadtimes*n.members)
      
      # in this case, must use psl.melted instead of psleuFull. Both are already in anomalies:
      # (problem: psl.melted depends on the startdate and leadtime, psleuFull no)
      pslmat <- unname(acast(psl.melted, Member + StartYear + LeadDay ~ Lat ~ Lon)) 
      #pslmat <- unname(acast(melt(pslPeriod[1,1:2,,,,, drop=FALSE],varnames=c("Exp","Member","StartYear","LeadDay","Lat","Lon")), Member ~ StartYear + LeadDay ~ Lat ~ Lon))
      
      pslwr1 <- pslmat[wr1,,, drop=F]
      pslwr2 <- pslmat[wr2,,, drop=F]
      pslwr3 <- pslmat[wr3,,, drop=F]
      pslwr4 <- pslmat[wr4,,, drop=F]

      pslwr1mean <- apply(pslwr1, c(2,3), mean, na.rm=T)
      pslwr2mean <- apply(pslwr2, c(2,3), mean, na.rm=T)
      pslwr3mean <- apply(pslwr3, c(2,3), mean, na.rm=T)
      pslwr4mean <- apply(pslwr4, c(2,3), mean, na.rm=T)
      
      rm(pslwr1,pslwr2,pslwr3,pslwr4)
      rm(pslmat)
      gc()

      # save all the data necessary to draw the graphs (but not the impact maps)
      save(lon, lon.max, lat, psl, psl.name, my.period, p, workdir,rean.name,fields.name,var.num,var.name,var.name.full,var.unit,year.start,year.end, WR.period, pslwr1mean, pslwr2mean,pslwr3mean,pslwr4mean,wr1y,wr2y,wr3y,wr4y,n.leadtimes,cluster.sequence,file=paste0(workdir,"/",fields.name,"_",var.name[var.num],"_",my.period[p],"_leadmonth_",lead.month,"_mapdata.RData"))
      
      rm(pslwr1mean,pslwr2mean,pslwr3mean,pslwr4mean)
      gc()
   
      #pslwr1meanPartial <- apply(pslwr1, c(1,3,4), mean, na.rm=T)
      #pslwr2meanPartial <- apply(pslwr2, c(1,3,4), mean, na.rm=T)
       #pslwr3meanPartial <- apply(pslwr3, c(1,3,4), mean, na.rm=T)
      #pslwr4meanPartial <- apply(pslwr4, c(1,3,4), mean, na.rm=T)

      #PlotEquiMap(rescale(pslwr1meanPartial[1,,],my.brks[1],tail(my.brks,1)), lon, lat, filled.continents=FALSE, brks=my.brks, cols=my.cols, sizetit=1.2)
      #PlotEquiMap(rescale(pslwr1meanPartial[2,,],my.brks[1],tail(my.brks,1)), lon, lat, filled.continents=FALSE, brks=my.brks, cols=my.cols, sizetit=1.2)

      #n=0
      #for(sy in 1:32) {
      #    for(ld in 1:28){
      #       if(my.cluster.array[[p]][ld,sy] == 1){
      #          n=n+1
      #          if(n == 1) {temp <- pslPeriod[1,2,sy,ld,,]} 
      #          temp <- temp + pslPeriod[1,2,sy,ld,,]
      #       }
      #    }
      #}
      #PlotEquiMap(rescale(temp/n,my.brks[1],tail(my.brks,1)), lon, lat, filled.continents=FALSE, brks=my.brks, cols=my.cols, sizetit=1.2, drawleg=F)
      
      #pslwr1meanPartial <- apply(pslmat[,wr1,,,drop=F], c(1,3,4), mean, na.rm=T)
      #pslwr2meanPartial <- apply(pslmat[,wr2,,,drop=F], c(1,3,4), mean, na.rm=T)
      #pslwr3meanPartial <- apply(pslmat[,wr3,,,drop=F], c(1,3,4), mean, na.rm=T)
      #pslwr4meanPartial <- apply(pslmat[,wr4,,,drop=F], c(1,3,4), mean, na.rm=T)
      
    }

    # for the monthly system, the time order is always:  year1day1, year2day1, ... , yearNday1, year1day2, year2day2, ..., yearNday2, etc.:
    if(fields.name == ECMWF_monthly.name) {
       if(fields.name == ECMWF_monthly.name){
       my.startdates.period <- months.period(forecast.year,mes,day,p)

       #if(p == 13) my.startdates.period <- my.startdates.period[-2]  # remove the second startdate because it is broken
       
       days.period <- period.length <- list()
       for(startdate in my.startdates.period){
          days.period[[p]] <- c(days.period[[p]], (1+(startdate-1)*(year.end-year.start+1)):(startdate*(year.end-year.start+1)))
       }
       period.length[[p]] <- length(my.startdates.period) # number of Thusdays in the period
      }

      wr1y <- wr2y <- wr3y <-wr4y <- c()
      wr1y[y-year.start+1] <- length(which(cluster.sequence[seq((y-year.start+1),length(cluster.sequence), n.years)] == 1))
      wr2y[y-year.start+1] <- length(which(cluster.sequence[seq((y-year.start+1),length(cluster.sequence), n.years)] == 2))
      wr3y[y-year.start+1] <- length(which(cluster.sequence[seq((y-year.start+1),length(cluster.sequence), n.years)] == 3))
      wr4y[y-year.start+1] <- length(which(cluster.sequence[seq((y-year.start+1),length(cluster.sequence), n.years)] == 4))
    }

} # close the for loop on 'p' 




    
